#include "stdafx.h"
#include <windows.h>
#include <functional>
#include "header/cudebug.h"
#include "header/TestSuite.h"
#include "header/TestCase.h"
#include "header/TestResult.h"
#include "header/TestRunner.h"

#include "dixsmartptr.h"
using namespace Dix;

USING_NAMESPACE_CPPUNIT

class CSample {
public:
	int a;
public:
	CSample() {}
	CSample( int a ) : a( a ) {}
	virtual ~CSample() {}
};

class CHoge : public CSample {
public:
	int b;
	CHoge() {}
	CHoge( int a, int b ) : CSample( a ), b( b ) {}
	virtual ~CHoge() {}
};

#define TEST_ASSERT_EQPTR( expect, value ) TEST_ASSERT_EQUALS( ( int )( expect ), ( int )( value ) );

class SPTest : public TestCase {

	TEST_CASE_COMMON( SPTest )

	void testConstruct() {
		// P
		sp< CSample > sp1( new CSample( 100 ) );
		TEST_ASSERT_EQUALS( false, sp1.IsAry() );
		TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
		TEST_ASSERT_EQUALS( 1, sp1.GetWeakNum() );

		// z
		sp< CSample > sp2( new CSample[ 2 ], true );
		TEST_ASSERT_EQUALS( true, sp2.IsAry() );
		TEST_ASSERT_EQUALS( 1, sp2.GetRefNum() );
		TEST_ASSERT_EQUALS( 1, sp2.GetWeakNum() );

		// Rs[RXgN^
		sp< CSample > sp3( sp1 );
		TEST_ASSERT_EQUALS( false, sp3.IsAry() );
		TEST_ASSERT_EQUALS( 2, sp3.GetRefNum() );
		TEST_ASSERT_EQUALS( 2, sp3.GetWeakNum() );

		sp< CSample > sp4( sp2 );
		TEST_ASSERT_EQUALS( true, sp4.IsAry() );
		TEST_ASSERT_EQUALS( 2, sp4.GetRefNum() );
		TEST_ASSERT_EQUALS( 2, sp4.GetWeakNum() );

		sp< CSample > sp5;
		TEST_ASSERT_EQUALS( false, sp5.IsAry() );
		TEST_ASSERT_EQUALS( 0, sp5.GetRefNum() );
		TEST_ASSERT_EQUALS( 0, sp5.GetWeakNum() );

		sp< CSample > sp6( 0 );	// NULL
		TEST_ASSERT_EQUALS( false, sp6.IsAry() );
		TEST_ASSERT_EQUALS( 0, sp6.GetRefNum() );
		TEST_ASSERT_EQUALS( 0, sp6.GetWeakNum() );

		sp< CSample > sp7( sp5 );	// sp
		TEST_ASSERT_EQUALS( false, sp7.IsAry() );
		TEST_ASSERT_EQUALS( 0, sp7.GetRefNum() );
		TEST_ASSERT_EQUALS( 0, sp7.GetWeakNum() );
	}

	void testOperator() {
		// P̖I
		CSample *obj1 = new CSample( 100 );
		sp< CSample > sp1( obj1 );
		sp< CSample > sp2;
		sp2 = sp1;
		TEST_ASSERT_EQUALS( false, sp2.IsAry() );
		TEST_ASSERT_EQUALS( 2, sp2.GetRefNum() );
		TEST_ASSERT_EQUALS( 2, sp2.GetWeakNum() );
		TEST_ASSERT_EQPTR( obj1, sp1.GetPtr() );
		TEST_ASSERT_EQPTR( obj1, sp2.GetPtr() );

		// z񖾎I
		CSample *obj2 = new CSample[ 2 ];
		obj2[ 0 ].a = 100;
		obj2[ 1 ].a = 200;
		sp< CSample > sp3( obj2, true );
		sp< CSample > sp4;
		sp4 = sp3;
		TEST_ASSERT_EQUALS( true, sp4.IsAry() );
		TEST_ASSERT_EQUALS( 2, sp4.GetRefNum() );
		TEST_ASSERT_EQUALS( 2, sp4.GetWeakNum() );
		TEST_ASSERT_EQPTR( obj2, sp3.GetPtr() );
		TEST_ASSERT_EQPTR( obj2, sp4.GetPtr() );

		// P̓sp
		CSample *obj3 = new CSample( 100 );
		sp< CSample > sp5( obj3 );
		sp5 = sp5;
		TEST_ASSERT_EQUALS( false, sp5.IsAry() );
		TEST_ASSERT_EQUALS( 1, sp5.GetRefNum() );
		TEST_ASSERT_EQUALS( 1, sp5.GetWeakNum() );
		TEST_ASSERT_EQPTR( obj3, sp5.GetPtr() );

		// z񓯂sp
		CSample *obj4 = new CSample[ 2 ];
		obj4[ 0 ].a = 100;
		obj4[ 1 ].a = 200;
		sp< CSample > sp6( obj4, true );
		sp6 = sp6;
		TEST_ASSERT_EQUALS( true, sp6.IsAry() );
		TEST_ASSERT_EQUALS( 1, sp6.GetRefNum() );
		TEST_ASSERT_EQUALS( 1, sp6.GetWeakNum() );
		TEST_ASSERT_EQPTR( obj4, sp6.GetPtr() );

		// P̖I
		{
			sp< CSample > sp1;
			sp< CSample > sp2;
			sp2 = sp1;
			TEST_ASSERT_EQUALS( false, sp2.IsAry() );
			TEST_ASSERT_EQUALS( 0, sp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, sp2.GetWeakNum() );
			TEST_ASSERT_EQPTR( 0, sp1.GetPtr() );
			TEST_ASSERT_EQPTR( 0, sp2.GetPtr() );
		}

		// P̖IiLȒlmj
		{
			CSample *obj1 = new CSample( 100 );
			CSample *obj2 = new CSample( 200 );
			sp< CSample > sp1( obj1 );
			sp< CSample > sp2( obj2 );
			sp2 = sp1;
			TEST_ASSERT_EQUALS( false, sp1.IsAry() );
			TEST_ASSERT_EQUALS( 2, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( false, sp2.IsAry() );
			TEST_ASSERT_EQUALS( 2, sp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, sp2.GetWeakNum() );
			TEST_ASSERT_EQPTR( obj1, sp1.GetPtr() );
			TEST_ASSERT_EQPTR( obj1, sp2.GetPtr() );
		}

		// P̖INULL
		{
			sp< CSample > sp1;
			sp1 = 0;
			TEST_ASSERT_EQUALS( false, sp1.IsAry() );
			TEST_ASSERT_EQUALS( 0, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, sp1.GetWeakNum() );
			TEST_ASSERT_EQPTR( 0, sp1.GetPtr() );
		}

		// *Zq
		{
			CSample *obj1 = new CSample( 100 );
			sp< CSample > sp1( obj1 );
			TEST_ASSERT_EQUALS( 100, ( *sp1 ).a );
		}

		// ->oIZq
		{
			CSample *obj1 = new CSample( 100 );
			sp< CSample > sp1( obj1 );
			TEST_ASSERT_EQUALS( 100, sp1->a );
		}

		// ==rZq
		{
			CSample *obj1 = new CSample( 100 );
			CSample *obj2 = new CSample( 200 );
			sp< CSample > sp1( obj1 );
			sp< CSample > sp2( obj2 );
			sp< CSample > sp3 = sp1;
			sp< CSample > sp4;
			TEST_ASSERT_EQUALS( false, sp1 == sp2 );
			TEST_ASSERT_EQUALS( true, sp1 == sp1 );
			TEST_ASSERT_EQUALS( true, sp3 == sp1 );
			TEST_ASSERT_EQUALS( true, sp3 == sp1.GetPtr() );
			TEST_ASSERT_EQUALS( false, sp3 == sp2.GetPtr() );
			TEST_ASSERT_EQUALS( false, sp1 == 0 );
			TEST_ASSERT_EQUALS( true, sp4 == 0 );
		}

		// !=rZq
		{
			CSample *obj1 = new CSample( 100 );
			CSample *obj2 = new CSample( 200 );
			sp< CSample > sp1( obj1 );
			sp< CSample > sp2( obj2 );
			sp< CSample > sp3 = sp1;
			sp< CSample > sp4;
			TEST_ASSERT_EQUALS( true, sp1 != sp2 );
			TEST_ASSERT_EQUALS( false, sp1 != sp1 );
			TEST_ASSERT_EQUALS( false, sp3 != sp1 );
			TEST_ASSERT_EQUALS( false, sp3 != sp1.GetPtr() );
			TEST_ASSERT_EQUALS( true, sp3 != sp2.GetPtr() );
			TEST_ASSERT_EQUALS( true, sp1 != 0 );
			TEST_ASSERT_EQUALS( false, sp4 != 0 );
		}

		// zQƉZq
		{
			CSample *obj1 = new CSample[ 3 ];
			obj1[ 0 ].a = 100;
			obj1[ 1 ].a = 200;
			obj1[ 2 ].a = 300;
			sp< CSample > sp1( obj1, true );
			TEST_ASSERT_EQUALS( 100, sp1[ 0 ].a );
			TEST_ASSERT_EQUALS( 200, sp1[ 1 ].a );
			TEST_ASSERT_EQUALS( 300, sp1[ 2 ].a );

			// P̂ւ̔zANZX͏ɗvf0̒lԂ
			CSample *obj2 = new CSample( 1000 );
			sp< CSample > sp2( obj2 );
			TEST_ASSERT_EQUALS( 1000, sp2[ 0 ].a );
			TEST_ASSERT_EQUALS( 1000, sp2[ 1 ].a );
			TEST_ASSERT_EQUALS( 1000, sp2[ 2 ].a );
		}

		// |C^Zq
		{
			CSample *obj1 = new CSample[ 3 ];
			obj1[ 0 ].a = 100;
			obj1[ 1 ].a = 200;
			obj1[ 2 ].a = 300;
			sp< CSample > sp1( obj1, true );
			TEST_ASSERT_EQUALS( 100, ( sp1 + 0 )->a );
			TEST_ASSERT_EQUALS( 200, ( sp1 + 1 )->a );
			TEST_ASSERT_EQUALS( 300, ( sp1 + 2 )->a );

			// P̂ւ̃|C^Z͏ɐ擪|C^Ԃ
			CSample *obj2 = new CSample( 1000 );
			sp< CSample > sp2( obj2 );
			TEST_ASSERT_EQUALS( 1000, ( sp2 + 0 )->a );
			TEST_ASSERT_EQUALS( 1000, ( sp2 + 1 )->a );
			TEST_ASSERT_EQUALS( 1000, ( sp2 + 2 )->a );
		}
	}

	void testGetVal() {
		// P
		{
			CSample *obj1 = new CSample( 100 );
			sp< CSample > sp1( obj1 );
			TEST_ASSERT_EQUALS( 100, sp1.At().a );
			TEST_ASSERT_EQUALS( 100, sp1.At( 1 ).a );
		}

		// z
		{
			CSample *obj1 = new CSample[ 3 ];
			obj1[ 0 ].a = 100;
			obj1[ 1 ].a = 200;
			obj1[ 2 ].a = 300;
			sp< CSample > sp1( obj1, true );
			TEST_ASSERT_EQUALS( 100, sp1.At( 0 ).a );
			TEST_ASSERT_EQUALS( 200, sp1.At( 1 ).a );
			TEST_ASSERT_EQUALS( 300, sp1.At( 2 ).a );
		}
	}

	void testDownCastCopy() {
		// _ELXg
		{
			// 
			sp< CSample > sp1( new CHoge( 100, 999 ) );
			sp< CHoge > spHoge;
			TEST_ASSERT_EQUALS( true, spHoge.DownCast( sp1 ) );
			TEST_ASSERT_EQUALS( 100, spHoge->a );
			TEST_ASSERT_EQUALS( 999, spHoge->b );
		}
		{
			// z_ELXgiT|[gȂj
			CHoge *ary = new CHoge[ 2 ];
			ary[ 0 ].a = 100;
			ary[ 0 ].b = 999;
			ary[ 1 ].a = 200;
			ary[ 1 ].b = 1999;
			sp< CSample > sp1( ary, true );
			sp< CHoge > spHoge;
			TEST_ASSERT_EQUALS( false, spHoge.DownCast( sp1 ) );
		}
	}

	void testSwap() {
		// |C^
		{
			sp< CSample > sp1( new CSample( 100 ) );
			sp< CSample > sp2( new CSample( 200 ) );
			sp1.SwapPtr( sp2 );
			TEST_ASSERT_EQUALS( 200, sp1->a );
			TEST_ASSERT_EQUALS( 100, sp2->a );
		}
		// |C^QƃJEg
		//  ĂQƃJE^͕ςȂ
		{
			sp< CSample > sp1( new CSample( 100 ) );
			sp< CSample > sp2( new CSample( 200 ) );
			sp< CSample > sp3 = sp1;
			sp< CSample > sp4 = sp1;
			TEST_ASSERT_EQUALS( 100, sp1->a );
			TEST_ASSERT_EQUALS( 3, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 200, sp2->a );
			TEST_ASSERT_EQUALS( 1, sp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 1, sp2.GetWeakNum() );
			sp1.SwapPtr( sp2 );
			TEST_ASSERT_EQUALS( 200, sp1->a );
			TEST_ASSERT_EQUALS( 3, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 100, sp2->a );
			TEST_ASSERT_EQUALS( 1, sp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 1, sp2.GetWeakNum() );
		}
		// |C^iṔ̔zj
		{
			sp< CSample > sp1( new CSample( 100 ) );
			CSample *obj2 = new CSample[ 2 ];
			obj2[ 0 ].a = 200;
			obj2[ 1 ].a = 300;
			sp< CSample > sp2( obj2, true );
			TEST_ASSERT_EQUALS( false, sp1.IsAry() );
			TEST_ASSERT_EQUALS( 100, sp1[ 0 ].a );
			TEST_ASSERT_EQUALS( true, sp2.IsAry() );
			TEST_ASSERT_EQUALS( 200, sp2[ 0 ].a );
			TEST_ASSERT_EQUALS( 300, sp2[ 1 ].a );
			sp1.SwapPtr( sp2 );
			TEST_ASSERT_EQUALS( true, sp1.IsAry() );
			TEST_ASSERT_EQUALS( 200, sp1[ 0 ].a );
			TEST_ASSERT_EQUALS( 300, sp1[ 1 ].a );
			TEST_ASSERT_EQUALS( false, sp2.IsAry() );
			TEST_ASSERT_EQUALS( 100, sp2[ 0 ].a );
		}
	}

public:
	static Test* suite() {
		TestSuite* suite = new TestSuite( "SPTest" );
		suite->addTest( new TEST_CASE( SPTest, testConstruct ) );
		suite->addTest( new TEST_CASE( SPTest, testOperator ) );
		suite->addTest( new TEST_CASE( SPTest, testGetVal ) );
		suite->addTest( new TEST_CASE( SPTest, testDownCastCopy ) );
		suite->addTest( new TEST_CASE( SPTest, testSwap ) );
		return suite;
	}
};


class WPTest : public TestCase {

	TEST_CASE_COMMON( WPTest )

		void testConstruct() {
		// P
		{
			sp< CSample > sp1( new CSample( 100 ) );
			wp< CSample > wp1( sp1 );
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 100, wp1->a );
		}

		// z
		{
			sp< CSample > sp1( new CSample[ 2 ], true );
			wp< CSample > wp1( sp1 );
			TEST_ASSERT_EQUALS( true, sp1.IsAry() );
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( true, wp1.IsAry() );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp1.GetWeakNum() );
		}

		// Rs[RXgN^
		{
			sp< CSample > sp1( new CSample( 100 ) );
			wp< CSample > wp1( sp1 );
			wp< CSample > wp2;
			wp2 = sp1;
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, wp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 1, wp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, wp2.GetWeakNum() );
			TEST_ASSERT_EQUALS( 100, wp2->a );
		}

		// NULL
		{
			wp< CSample > wp1( 0 );
			TEST_ASSERT_EQUALS( 0, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, wp1.GetWeakNum() );
		}

		// NULL sp
		{
			sp< CSample > sp1( 0 );	// NULL
			wp< CSample > wp1( sp1 );
			TEST_ASSERT_EQUALS( 0, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 0, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, wp1.GetWeakNum() );
		}
	}

	void testRefCounter() {
		// P̎QƃJE^ϓ
		{
			sp< CSample > sp1( new CSample( 100 ) );

			wp< CSample > wp1;
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 1, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 0, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, wp1.GetWeakNum() );

			wp1 = sp1;
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp1.GetWeakNum() );
			{
				wp< CSample > wp2( sp1 );
				TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
				TEST_ASSERT_EQUALS( 3, sp1.GetWeakNum() );
				TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
				TEST_ASSERT_EQUALS( 3, wp1.GetWeakNum() );
				TEST_ASSERT_EQUALS( 1, wp2.GetRefNum() );
				TEST_ASSERT_EQUALS( 3, wp2.GetWeakNum() );

				{
					sp< CSample > sp2( sp1 );
					TEST_ASSERT_EQUALS( 2, sp1.GetRefNum() );
					TEST_ASSERT_EQUALS( 4, sp1.GetWeakNum() );
					TEST_ASSERT_EQUALS( 2, wp1.GetRefNum() );
					TEST_ASSERT_EQUALS( 4, wp1.GetWeakNum() );
					TEST_ASSERT_EQUALS( 2, wp2.GetRefNum() );
					TEST_ASSERT_EQUALS( 4, wp2.GetWeakNum() );
				}
			}
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, sp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp1.GetWeakNum() );
		}
	}

	void testOperator() {

		// 
		{
			sp< CHoge > sp1( new CHoge( 100, 200 ) );
			{
				std::function< void( wp< CSample > ) > func = [ this, sp1 ]( wp< CSample > wp1 ) {
					TEST_ASSERT_EQUALS( 2, sp1.GetRefNum() );	// functioñXR[vێ邽2
					TEST_ASSERT_EQUALS( 4, sp1.GetWeakNum() );
					TEST_ASSERT_EQUALS( 2, wp1.GetRefNum() );
					TEST_ASSERT_EQUALS( 4, wp1.GetWeakNum() );
				};
				func( sp1 );
				TEST_ASSERT_EQUALS( 2, sp1.GetRefNum() );
				TEST_ASSERT_EQUALS( 2, sp1.GetWeakNum() );
			}
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 1, sp1.GetWeakNum() );
		}

		// 
		{
			sp< CHoge > sp1;
			{
				std::function< void( wp< CSample > ) > func = [this, sp1]( wp< CSample > wp1 ) {
					TEST_ASSERT_EQUALS( 0, sp1.GetRefNum() );	// functioñXR[vێ邽2
					TEST_ASSERT_EQUALS( 0, sp1.GetWeakNum() );
					TEST_ASSERT_EQUALS( 0, wp1.GetRefNum() );
					TEST_ASSERT_EQUALS( 0, wp1.GetWeakNum() );
				};
				func( sp1 );
				TEST_ASSERT_EQUALS( 0, sp1.GetRefNum() );
				TEST_ASSERT_EQUALS( 0, sp1.GetWeakNum() );
			}
			TEST_ASSERT_EQUALS( 0, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, sp1.GetWeakNum() );
		}

		// wp
		{
			sp< CHoge > sp1( new CHoge( 100, 200 ) );
			{
				std::function< void( wp< CSample > ) > func = [this, sp1]( wp< CSample > wp1 ) {
					TEST_ASSERT_EQUALS( 2, sp1.GetRefNum() );	// functioñXR[vێ邽2
					TEST_ASSERT_EQUALS( 5, sp1.GetWeakNum() );
					TEST_ASSERT_EQUALS( 2, wp1.GetRefNum() );
					TEST_ASSERT_EQUALS( 5, wp1.GetWeakNum() );
				};
				wp< CSample > wp2( sp1 );
				func( wp2 );
				TEST_ASSERT_EQUALS( 2, sp1.GetRefNum() );
				TEST_ASSERT_EQUALS( 3, sp1.GetWeakNum() );
			}
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 1, sp1.GetWeakNum() );
		}

		// wp Null
		{
			sp< CHoge > sp1;
			wp< CSample > wp1;
			wp1 = sp1;
			TEST_ASSERT_EQUALS( 0, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 0, sp1.GetWeakNum() );
		}

		// wpZq
		{
			sp< CHoge > sp1( new CHoge( 100, 200 ) );
			wp< CSample > wp1;
			wp1 = sp1;
			TEST_ASSERT_EQUALS( 1, sp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, sp1.GetWeakNum() );
		}

		// wpZqiȑj
		{
			sp< CHoge > sp1( new CHoge( 100, 200 ) );
			wp< CSample > wp1( sp1 );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp1.GetWeakNum() );
			wp1 = wp1;
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp1.GetWeakNum() );
		}

		// ==rZq
		{
			sp< CHoge > sp1( new CHoge( 100, 200 ) );
			wp< CHoge > wpHoge( sp1 );
			wp< CSample > wp1( sp1 );
			wp< CSample > wpNull;
			TEST_ASSERT_EQUALS( true, wp1 == sp1 );
			TEST_ASSERT_EQUALS( true, wpHoge == sp1 );
			TEST_ASSERT_EQUALS( true, wpNull == 0 );
			TEST_ASSERT_EQUALS( false, wpNull == wpHoge );
			TEST_ASSERT_EQUALS( false, wpHoge == wpNull );
		}

		// !=rZq
		{
			sp< CHoge > sp1( new CHoge( 100, 200 ) );
			wp< CHoge > wpHoge( sp1 );
			wp< CSample > wp1( sp1 );
			wp< CSample > wpNull;
			TEST_ASSERT_EQUALS( false, wp1 != sp1 );
			TEST_ASSERT_EQUALS( false, wpHoge != sp1 );
			TEST_ASSERT_EQUALS( false, wpNull != 0 );
			TEST_ASSERT_EQUALS( true, wpNull != wpHoge );
			TEST_ASSERT_EQUALS( true, wpHoge != wpNull );
		}


	}

	void testMethods() {

		// X}[g|C^쐬
		{
			sp< CHoge > sp1( new CHoge( 100, 200 ) );
			wp< CHoge > wp1( sp1 );
			sp< CHoge > sp2 = wp1.GetSmartPtr();
			TEST_ASSERT_EQUALS( true, sp1 == sp2 );
			TEST_ASSERT_EQUALS( 2, sp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, sp2.GetWeakNum() );
		}

		// sp_ELXgRs[
		{
			sp< CSample > sp1( new CHoge( 100, 200 ) );
			wp< CHoge > wp1;
			TEST_ASSERT_EQUALS( true, wp1.DownCast( sp1 ) );
			TEST_ASSERT_EQUALS( true, wp1 == sp1 );
			TEST_ASSERT_EQUALS( 100, wp1->a );
			TEST_ASSERT_EQUALS( 200, wp1->b );

			wp< CSample > wp2( sp1 );
			TEST_ASSERT_EQUALS( true, wp1.DownCast( wp2 ) );
		}

		// SwapiṔj
		{
			sp< CSample > sp1( new CSample( 100 ) );
			sp< CSample > sp2( new CSample( 200 ) );
			wp< CSample > wp1( sp1 );
			wp< CSample > wpdummy( wp1 );	// QƃJE^3ɂ邽
			wp< CSample > wp2( sp2 );
			TEST_ASSERT_EQUALS( 100, sp1->a );
			TEST_ASSERT_EQUALS( 100, wp1->a );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, wp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 200, sp2->a );
			TEST_ASSERT_EQUALS( 200, wp2->a );
			TEST_ASSERT_EQUALS( 1, wp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp2.GetWeakNum() );

			wp1.SwapPtr( wp2 );

			TEST_ASSERT_EQUALS( 200, sp1->a );
			TEST_ASSERT_EQUALS( 200, wp1->a );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, wp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 100, sp2->a );
			TEST_ASSERT_EQUALS( 100, wp2->a );
			TEST_ASSERT_EQUALS( 1, wp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp2.GetWeakNum() );
		}

		// SwapiṔ̔zj
		{
			CSample *ary = new CSample[ 2 ];
			ary[ 0 ].a = 200;
			ary[ 1 ].a = 300;
			sp< CSample > sp1( new CSample( 100 ) );
			sp< CSample > sp2( ary, true );
			wp< CSample > wp1( sp1 );
			wp< CSample > wpdummy( wp1 );	// QƃJE^3ɂ邽
			wp< CSample > wp2( sp2 );
			TEST_ASSERT_EQUALS( 100, sp1->a );
			TEST_ASSERT_EQUALS( 100, wp1->a );
			TEST_ASSERT_EQUALS( 100, sp1[ 1 ].a );
			TEST_ASSERT_EQUALS( 100, wp1[ 1 ].a );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, wp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 200, sp2->a );
			TEST_ASSERT_EQUALS( 200, wp2->a );
			TEST_ASSERT_EQUALS( 300, sp2[ 1 ].a );
			TEST_ASSERT_EQUALS( 300, wp2[ 1 ].a );
			TEST_ASSERT_EQUALS( 1, wp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp2.GetWeakNum() );

			wp1.SwapPtr( wp2 );

			TEST_ASSERT_EQUALS( 200, sp1->a );
			TEST_ASSERT_EQUALS( 200, wp1->a );
			TEST_ASSERT_EQUALS( 300, sp1[ 1 ].a );
			TEST_ASSERT_EQUALS( 300, wp1[ 1 ].a );
			TEST_ASSERT_EQUALS( 1, wp1.GetRefNum() );
			TEST_ASSERT_EQUALS( 3, wp1.GetWeakNum() );
			TEST_ASSERT_EQUALS( 100, sp2->a );
			TEST_ASSERT_EQUALS( 100, wp2->a );
			TEST_ASSERT_EQUALS( 100, sp2[ 1 ].a );
			TEST_ASSERT_EQUALS( 100, wp2[ 1 ].a );
			TEST_ASSERT_EQUALS( 1, wp2.GetRefNum() );
			TEST_ASSERT_EQUALS( 2, wp2.GetWeakNum() );
		}

		// ݂̂ĂH
		{
			sp< CSample > sp1( new CSample( 100 ) );
			sp< CSample > sp2;
			wp< CSample > wp1( sp1 );
			wp< CSample > wp2( sp2 );
			wp< CSample > wp3;
			TEST_ASSERT_EQUALS( true, wp1.IsExist() );
			TEST_ASSERT_EQUALS( false, wp2.IsExist() );
			TEST_ASSERT_EQUALS( false, wp3.IsExist() );
		}
	}

public:
	static Test* suite() {
		TestSuite* suite = new TestSuite( "SPTest" );
		suite->addTest( new TEST_CASE( WPTest, testConstruct ) );
		suite->addTest( new TEST_CASE( WPTest, testRefCounter ) );
		suite->addTest( new TEST_CASE( WPTest, testOperator ) );
		suite->addTest( new TEST_CASE( WPTest, testMethods ) );
		return suite;
	}
};


int main( int argc, char* argv[] ) {

	TestRunner runner;
	runner.addTest( "SPTest", SPTest::suite() );
	runner.addTest( "WPTest", WPTest::suite() );

	return runner.run( argc, argv );
}